#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"

#define SYS_LAYOUT_DIR "/usr/share/matchbox-keyboard"
#define USR_LAYOUT_SUFFIX ".matchbox/keyboard.d"

static void fill_system_list (GtkListStore ** list)
{
  GtkTreeIter iter;
  DIR * dir;
  struct dirent * de;
  char name_buffer[256];
  struct stat st;
  /* Append a row and fill in some data */
  dir = opendir(SYS_LAYOUT_DIR);
  if(!dir) {
    perror("opendir(" SYS_LAYOUT_DIR ")");
    exit(1);
  }
  while((de = readdir(dir))) {
    strcpy(name_buffer, SYS_LAYOUT_DIR);
    strcat(name_buffer, "/");
    strcat(name_buffer, de->d_name);
    stat(name_buffer, &st);
    if(strlen(de->d_name)>4 && !strcmp(de->d_name+strlen(de->d_name)-4,".xml")) {
	if(S_ISREG(st.st_mode)) {
    	    gtk_list_store_append (*list, &iter);
    	    gtk_list_store_set (*list, &iter, 0, de->d_name, -1);
	}
    }
  }
  closedir(dir);
}

static GtkWidget *user_layouts_view;
static GtkWidget *system_layouts_view;

static gboolean
rm_func_sys (GtkTreeModel *model, GtkTreePath *path,
         GtkTreeIter *iter, gpointer user_data)
{
 gchar * nameusr = (gchar *) user_data;
 gchar * namesys;
 gtk_tree_model_get (model, iter, 0, &namesys, -1);
 g_assert(nameusr); g_assert(namesys);
 if(!strncmp(nameusr, namesys, PATH_MAX)) {
      gtk_list_store_remove(GTK_LIST_STORE(model), iter);
 }
 return FALSE;
}
static gboolean
rm_func (GtkTreeModel *model, GtkTreePath *path,
         GtkTreeIter *iter, gpointer user_data)
{
  gchar * name;
  GtkTreeModel * model1;
  gtk_tree_model_get (model, iter, 0, &name, -1);
  model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(system_layouts_view));
  gtk_tree_model_foreach(GTK_TREE_MODEL(model1), rm_func_sys, name);
  return FALSE;
}
static void clean_system_list(void)
{
 GtkTreeModel * model1;
 model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(user_layouts_view));
 gtk_tree_model_foreach(GTK_TREE_MODEL(model1), rm_func, NULL);
}

static void fill_user_list(GtkListStore ** list)
{
 GtkTreeIter iter;
 DIR * dir;
 struct dirent * de;
 struct stat st;
 char linkpath[PATH_MAX];
 char name_buffer[PATH_MAX];
 char * home_dir = getenv("HOME");
 g_assert(home_dir);
 strcpy(linkpath, home_dir);
 strcat(linkpath, "/");
 strcat(linkpath, USR_LAYOUT_SUFFIX);
 dir = opendir(linkpath);
 if(!dir) {
    perror("opendir");
    exit(1);
 }
  while((de = readdir(dir))) {
    strcpy(name_buffer, linkpath);
    strcat(name_buffer, "/");
    strcat(name_buffer, de->d_name);
    lstat(name_buffer, &st);
    if(strlen(de->d_name)>4 && !strcmp(de->d_name+strlen(de->d_name)-4,".xml")) {
	if((st.st_mode & S_IFLNK) == S_IFLNK && !stat(name_buffer, &st)) {
    	    gtk_list_store_append (*list, &iter);
    	    gtk_list_store_set (*list, &iter, 0, de->d_name, -1);
	}
    }
  }
  closedir(dir);
}

static gboolean
add_layout (GtkWidget *w, GtkWidget *t)
{
  GtkTreeSelection *sel;
  GtkTreeModel     *model, *model1;
  GtkTreePath      *path;
  GtkTreeIter	   iter, iter1;
  gchar            *layout = NULL; 

  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (system_layouts_view));

  if (gtk_tree_selection_get_selected (sel, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, 0, &layout, -1);
      model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(user_layouts_view));
      g_assert(model1);
      gtk_list_store_append (GTK_LIST_STORE(model1), &iter1);
      gtk_list_store_set (GTK_LIST_STORE(model1), &iter1, 0, layout, -1);
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
      g_free(layout);
    }

  return TRUE;
}


static gboolean
remove_layout (GtkWidget *w, GtkWidget *t)
{
  GtkTreeSelection *sel;
  GtkTreeModel     *model, *model1;
  GtkTreePath      *path;
  gchar            *layout = NULL; 
  GtkTreeIter	   iter, iter1;

  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (user_layouts_view));

  if (gtk_tree_selection_get_selected (sel, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, 0, &layout, -1);
      model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(system_layouts_view));
      g_assert(model1);
      gtk_list_store_append (GTK_LIST_STORE(model1), &iter1);
      gtk_list_store_set (GTK_LIST_STORE(model1), &iter1, 0, layout, -1);
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
      g_free(layout);
    }

  return TRUE;
}

static gboolean
ff_func (GtkTreeModel *model, GtkTreePath *path,
         GtkTreeIter *iter, gpointer user_data)
{
  gchar * name;
  char linkbuf[PATH_MAX];
  char filebuf[PATH_MAX];
  gchar * dir = (gchar *) user_data;
  gtk_tree_model_get (model, iter, 0, &name, -1);
  strcpy(linkbuf, dir);
  strcat(linkbuf, "/");
  strcat(linkbuf, name);
  strcpy(filebuf, SYS_LAYOUT_DIR);
  strcat(filebuf, "/");
  strcat(filebuf, name);
  unlink(linkbuf);
  if(symlink(filebuf, linkbuf) != 0) {
     perror("symlink");
     exit(1);
  }
  puts(linkbuf);
  g_free(name);
  return FALSE;
}

static void cleanup_dir(char * dirpath)
{
  DIR * dir;
  struct dirent * de;
  struct stat st;
  char name_buffer[PATH_MAX];
  char * home_dir = getenv("HOME");

  dir = opendir(dirpath);
  while((de = readdir(dir))) {
    strcpy(name_buffer, home_dir);
    strcat(name_buffer, "/");
    strcat(name_buffer, USR_LAYOUT_SUFFIX);
    strcat(name_buffer, "/");
    strcat(name_buffer, de->d_name);
    lstat(name_buffer, &st);
    if(strlen(de->d_name)>4 && !strcmp(de->d_name+strlen(de->d_name)-4,".xml")) {
	if((st.st_mode & S_IFLNK) == S_IFLNK && !stat(name_buffer, &st)) {
	    unlink(name_buffer);
	}
    }
  }
  closedir(dir);
}
static gboolean
make_links (GtkWidget *w, GtkWidget *t)
{
 char linkpath[PATH_MAX];
 char * home_dir = getenv("HOME");
 GtkTreeModel * model1;
 g_assert(home_dir);
 strcpy(linkpath, home_dir);
 strcat(linkpath, "/");
 strcat(linkpath, USR_LAYOUT_SUFFIX);
 cleanup_dir(linkpath);
 model1 = gtk_tree_view_get_model(GTK_TREE_VIEW(user_layouts_view));
 g_assert(model1);
 gtk_tree_model_foreach(GTK_TREE_MODEL(model1), ff_func, linkpath);
 
 gtk_main_quit();
 return TRUE;
}

GtkWidget*
create_layout_dialog (void)
{
  GtkWidget *layout_dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *dialog_hbox;
  GtkWidget *dialog_table;
  GtkWidget *user_layouts_scroll;
  GtkListStore *user_list_store;
  GtkWidget *system_layouts_scroll;
  GtkListStore *system_list_store;
  GtkWidget *add_layout_button;
  GtkWidget *remove_layout_button;
  GtkWidget *dialog_action_area1;
  GtkWidget *cancelbutton;
  GtkWidget *okbutton;
  GtkCellRenderer *renderer;


  layout_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width (GTK_CONTAINER (layout_dialog), 7);
  gtk_window_set_title (GTK_WINDOW (layout_dialog), _("Layouts"));
  gtk_window_set_modal (GTK_WINDOW (layout_dialog), TRUE);
//  gtk_window_set_type_hint (GTK_WINDOW (layout_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);

//  dialog_vbox = GTK_DIALOG (layout_dialog)->vbox;
//  gtk_widget_show (dialog_vbox);

  dialog_vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_add(GTK_CONTAINER(layout_dialog), dialog_vbox);

  dialog_table = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (dialog_table);
  gtk_box_pack_start (GTK_BOX (dialog_vbox), dialog_table, TRUE, TRUE, 0);

  user_list_store = gtk_list_store_new (1, G_TYPE_STRING);
  system_list_store = gtk_list_store_new (1, G_TYPE_STRING);
  fill_system_list(&system_list_store);
  fill_user_list(&user_list_store);
  system_layouts_scroll = gtk_scrolled_window_new(NULL, NULL);
  user_layouts_scroll = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (user_layouts_scroll),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (system_layouts_scroll),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  renderer = gtk_cell_renderer_text_new ();
  user_layouts_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(user_list_store));
  gtk_container_add(GTK_CONTAINER(user_layouts_scroll), user_layouts_view);
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (user_layouts_view),
                                               -1,      
                                               "User Layouts",  
                                               renderer,
                                               "text", 0,
                                               NULL);


  gtk_widget_show (user_layouts_scroll);
  gtk_widget_show (user_layouts_view);
  gtk_table_attach (GTK_TABLE (dialog_table), user_layouts_scroll, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_container_set_border_width (GTK_CONTAINER (user_layouts_scroll), 10);

  system_layouts_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(system_list_store));
  clean_system_list();
  gtk_container_add(GTK_CONTAINER(system_layouts_scroll), system_layouts_view);
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (system_layouts_view),
                                               -1,      
                                               "System Layouts",  
                                               renderer,
                                               "text", 0,
                                               NULL);
  gtk_widget_show (system_layouts_scroll);
  gtk_widget_show (system_layouts_view);
  gtk_table_attach (GTK_TABLE (dialog_table), system_layouts_scroll, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (system_layouts_scroll), 10);

  add_layout_button = gtk_button_new_with_mnemonic (_("<<"));
  gtk_widget_show (add_layout_button);
  gtk_table_attach (GTK_TABLE (dialog_table), add_layout_button, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  remove_layout_button = gtk_button_new_with_mnemonic (_(">>"));
  gtk_widget_show (remove_layout_button);
  gtk_table_attach (GTK_TABLE (dialog_table), remove_layout_button, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
/*
  dialog_action_area1 = GTK_DIALOG (layout_dialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (layout_dialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);
  gtk_button_set_focus_on_click (GTK_BUTTON (cancelbutton), FALSE);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (layout_dialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
*/
  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  okbutton = gtk_button_new_from_stock ("gtk-ok");
  dialog_hbox = gtk_hbox_new(FALSE, 2);
  gtk_widget_show (okbutton);
  gtk_widget_show (cancelbutton);
  gtk_box_pack_end(GTK_BOX(dialog_vbox), GTK_WIDGET(dialog_hbox), FALSE, TRUE, 0);
  gtk_box_pack_end(GTK_BOX(dialog_hbox), cancelbutton, FALSE, TRUE, 0);
  gtk_box_pack_end(GTK_BOX(dialog_hbox), okbutton, FALSE, TRUE, 0);
  gtk_button_set_focus_on_click (GTK_BUTTON (cancelbutton), FALSE);

  g_signal_connect (G_OBJECT (add_layout_button), "clicked", 
		    G_CALLBACK (add_layout), NULL);

  g_signal_connect (G_OBJECT (remove_layout_button), "clicked", 
		    G_CALLBACK (remove_layout), NULL);


  g_signal_connect (G_OBJECT (layout_dialog), "delete_event", 
		    G_CALLBACK (gtk_main_quit), NULL);

  g_signal_connect (G_OBJECT (cancelbutton), "clicked", 
		    G_CALLBACK (gtk_main_quit), NULL);

  g_signal_connect (G_OBJECT (okbutton), "clicked", 
		    G_CALLBACK (make_links), NULL);

  return layout_dialog;
}

